#include <stdlib.h>
#include "iostreamc"
#include "timec.h"

#include "gdal.h"

//===========================================================================
// This program was written to test the JPGDataset::IRasterIO optimizations
//
// The program reads the same raster a specified number of times.  But it
// does so by openning and closing the raster each time in order to simulate
// reading a different raster each iteration.  If the open and close were not
// executed each read, then the same raster data would probably be re-read
// from the cache each time through the loop.
//
// Testing with and without the optimization was executed with this raster:
// Driver: JPEG/JPEG JFIF
// Size is 1024, 512
// Coordinate System is `'
// Image Structure Metadata:
//   INTERLEAVE=PIXEL
//   COMPRESSION=JPEG
// Corner Coordinates:
// Upper Left  (    0.0,    0.0)
// Lower Left  (    0.0,  512.0)
// Upper Right ( 1024.0,    0.0)
// Lower Right ( 1024.0,  512.0)
// Center      (  512.0,  256.0)
// Band 1 Block=1024x1 Type=Byte, ColorInterp=Red
//   Image Structure Metadata:
//     COMPRESSION=JPEG
// Band 2 Block=1024x1 Type=Byte, ColorInterp=Green
//   Image Structure Metadata:
//     COMPRESSION=JPEG
// Band 3 Block=1024x1 Type=Byte, ColorInterp=Blue
//   Image Structure Metadata:
//     COMPRESSION=JPEG
//
// The optimized runs completed approximately 3 times as fast as the non-
// -opmitimzed runs.
//===========================================================================

//===========================================================================
inline double getSystemTime()
{
  struct timeval tp;
  gettimeofday (&tp, 0);
  return ((double)tp.tv_sec + ((double)tp.tv_usec * 1e-06));
}

//===========================================================================
int main(int argc, char* argv[])
{
  if(argc < 3)
  {
    cerr << "ERROR:  not enough arguments\n";
    return 1;
  }

  int numTimesToRead = atoi(argv[2]);

  // initializes GDAL
  GDALAllRegister();

  int i;
  double startTime = getSystemTime();

  for(i = 0; i < numTimesToRead; ++i)
  {
    GDALDatasetH image = GDALOpen(argv[1], GA_ReadOnly);
    if(image == NULL)
    {
      cerr << "ERROR:  unable to open (" << argv[1] << ") for reading\n";
      GDALDestroyDriverManager();
      return 1;
    }

    int xSize = GDALGetRasterXSize(image);
    int ySize = GDALGetRasterYSize(image);
    int numBands = GDALGetRasterCount(image);
    int numBytes = xSize * ySize * numBands;
    char* pixels = new char[numBytes];
    memset(pixels, 0xff, numBytes);

    // the following code assumes 8 bits per band value
    GDALDatasetRasterIO(image, GF_Read,
			0, 0, xSize, ySize,
			&(pixels[0]), xSize, ySize, GDT_Byte,
			numBands, NULL,
			numBands, xSize*numBands, 1);
    delete [] pixels;
    GDALClose(image);
  }

  double delta = getSystemTime() - startTime;
  GDALDestroyDriverManager();

  cerr << delta << "\n";
  return 0;
}
